------------------------------------------------------------------
-- Oracle Session-Funktionspackage
------------------------------------------------------------------
CREATE OR REPLACE PACKAGE BLV_SESSION IS
  
  FUNCTION GET_VERSION RETURN VARCHAR2;
  
  /** Session-Initialisierung 
   *  bergaben:
   *    sSession_ID: Eindeutiges Identifizierungs-Literal (GUID, '::SSID::', ...)
   *    nUserSID:    BALVI iP UserSID (S50.SID), welches beim Login ermittelt wurde
   *  Beschreibung:
   *     INIT ruft implizit auch CLEAR() auf, um ggf. alte Werte zu lschen.
   *     Fr die neue Session_ID wird der Kopfsatz mit dem CODE='SESSION' angelegt,
   *     VALUE    = NULL
   *     VALUE_DF = SYSDATE der Anlage (Session-Start)
   *     VALUE_DT = SYSDATE +1 (Session-Ende), wenn nicht explizit gelscht
   *     VALUE_N  = nUserSID
   **/
  PROCEDURE INIT (sSession_ID VARCHAR2, nUserSID NUMBER, sSessionName VARCHAR2 := NULL);
  
  /** is_Valid - berprfung, ob Session/User gltig ist
   *  bergaben:
   *    sSession_ID: Eindeutiges Identifizierungs-Literal
   *    nUserSID:    BALVI iP UserSID (S50.SID), welches beim Login ermittelt wurde
   *  Rckgaben: 
   *    0 = FALSE
   *    1 = TRUE
   *  Fallbeispiel:
   *    User ndert sich, Session_id bleibt (Benutzerwechsel) => INIT neu aufrufen
   *   begin
	 *     -- Session prfen, ob Konsistent
   *     if blv_session.is_valid('12345',2209001)=0 then
   *        -- INIT (SESSION_ID STRING, USER_SID INTEGER, [Session_NAME STRING]) 
   *        blv_session.init('12345',2209001,'Test');
   *        -- ADD_S07_PARAM(SESSION_ID STRING, USER_SID INTEGER, S07-CODE, S0701-CODE)
   *        blv_session.add_s07_param('12345',2209001,'mobil','mobil_import_path');
   *     end if;
   *   end; 
   **/
  FUNCTION is_Valid(sSession_ID VARCHAR2,nUserSID NUMBER) RETURN NUMBER;
  
  PROCEDURE CLEAR (sSession_ID VARCHAR2);
  
  /** ADD - Globale Hauptfunktion
   *  bergaben:
   *    sSession_ID: Eindeutiges Identifizierungs-Literal
   *    sCode:       Name des Session-Parameters (wird immer UPPER(WKD_SESSION.CODE) abgelegt)
   *    sValue:      Wert als String (in Spalte WKD_SESSION.VALUE)
   *    nValue:      Wert als Integer (in Spalte WKD_SESSION.VALUE_N)
   *    fValue:      Wert als Float (in Spalte WKD_SESSION.VALUE_F), kein "simples ADD" vorhanden
   *    dValueFrom:  Wert als Datum (in Salte WKD_SESSION.VALUE_DF)
   *    dValueTo:    Wert als Datum (in Salte WKD_SESSION.VALUE_DT)
   *    lfdnr:       Integer fr Sortier-Reihenfolgen, default 1
   *
   *  Es wird keine Prfung vorgenommen, ob INIT() ausgefhrt wurde!
   **/
  PROCEDURE ADD(sSession_ID VARCHAR2, sCode VARCHAR2, sValue VARCHAR2, nValue NUMBER, fValue FLOAT, dValueFrom DATE, dValueTo DATE, lfdnr NUMBER);
  
  /** 
   *  ADD - OVERLOAD-Methoden
   *
   *  Je nach bergebenem Datentyp entscheidet Oracle eigenstndig, 
   *  welche Overload-Methode verwendet wird. Um sich zu gehen, dass NULL korrekt bergeben wird,
   *  muss CAST( NULL AS TYP) in der Methode bergeben werden.
   *
   * Beispiel Cast String als Number oder umgekehrt:
   *   exec blv_session.add('12345','TEST1',CAST ('0101' AS NUMBER));
   *   exec blv_session.add('12345','TEST2',CAST (0101 AS VARCHAR2));
   **/
  PROCEDURE ADD(sSession_ID varchar2, sCode varchar2, sValue varchar2, nValue number, fValue float, dValueFrom date, dValueTo date);
  PROCEDURE ADD(sSession_ID varchar2, sCode varchar2, sValue varchar2);
  PROCEDURE ADD(sSession_ID varchar2, sCode varchar2, nValue number);
  /*PROCEDURE ADD(sSession_ID varchar2, sCode varchar2, fValue float);*/
  PROCEDURE ADD(sSession_ID varchar2, sCode varchar2, dValueFrom date, dValueTo date);

  /** UPD - Aktualisierung eines Wertes
   *
   *  bergaben:
   *    => Siehe ADD()
   *    addIfNull BOOLEAN = TRUE oder FALSE bergeben, default TRUE
   *  Besonderheiten:
   *    Die Funktion UPD() lscht alle nicht bergebenen Spalten
   *    aus dem Datensatz. Wenn der Datensatz mit dem entsprechenden CODE
   *    1.) NICHT EINDEUTIG => ROWCOUNT > 1 ist,
   *         => keine ndern
   *    2.) NICHT VORHANDEN => ROWCOUNT = 0 ist,
   *         => es wird ADD() aufgerufen, wenn addIfNull=TRUE 
   *
   *   Es werden KEINE OVERLOAD-Methoden zur Verfgung gestellt!
   *   Es gilt jedoch "verkrzte Form". Alle Parameter auer
   *      sSession_ID varchar2, sCode varchar2, sValue varchar2
   *   sind OPTIONAL, d.h. der festgelegte DEFAULT wird implizit angenommen!
   *   Beispiel:
   *      exec blv_session.upd('12345','TEST2',CAST (0101 AS VARCHAR2),0101);
   **/
  PROCEDURE UPD(sSession_ID varchar2, sCode varchar2, sValue varchar2, nValue number := NULL , fValue float := NULL , dValueFrom date := NULL, dValueTo date := NULL, lfdnr number := 1, addIfNull boolean := TRUE);
  
  /** DEL - Lschen aller Eintrge mit Name <sCode>
   *  bergaben:
   *    sSession_ID: Eindeutiges identifizierungs-Literal
   *    sCode:       Name des Session-Parameters
   *
   *  Es werden alle Werte mit dem entsprechenden CODE gelscht,
   *  wenn vorhanden. Um Sicherzustellen, dass ein CODE/WERTEPAAR 
   *  mit ADD() keine doppelten Eintrge erzeugt, sollte folgendermaen vorgegangen 
   *  werden:    
   *   BLV_SESSION.DEL(sSession_ID,'<CODE>');
   *   BLV_SESSION.ADD(sSession_ID,'<CODE>', <WERT>);
   **/
  PROCEDURE DEL(sSession_ID varchar2, sCode varchar2);
  
  /**  BALVI IP -Fachlogik-spezialfunktionen
   *   - ADD_ZB_FILTER wird implizit im INIT aufgerufen, erneutes Aufrufen setzt die entsprechenden Werte zurck
   *   - ADD_USER_INFO wird implizit im INIT aufgerufen, erneutes Aufrufen setzt die entsprechenden Werte zurck
   *   - ADD_S07_PARAM muss manuell aufgerufen werden.
   **/
   
  PROCEDURE ADD_ZB_FILTER(sSession_ID varchar2,nUserSID number);
  PROCEDURE ADD_USER_INFO(sSession_ID varchar2,nUserSID number);
  
  PROCEDURE ADD_S07_PARAM(sSession_ID varchar2 ,nUserSID number, sS07_CODE varchar2, sS0701_CODE varchar2, sCode varchar2 := NULL);
  
  /** GETter-Methoden fr Spezialflle 
   *  Alle GET_...-Methoden liefern nur einen Wert zurck, auch wenn in der Ergebnismenge
   *  mehr als ein Datensatz fr den CODE vorhanden ist. Es gibt keine Sortierung,
   *  d.h. das Ergebnis kann zuflig sein!
   * 
   *  Beispiel:
   *    exec blv_session.add('12345','GETTEST',1);
   *    exec blv_session.add('12345','GETTEST',2);
   *    exec blv_session.add('12345','GETTEST',3);
   *    select blv_session.get_value_n('12345','GETTEST') from DUAL;
   *
   **/
  
  FUNCTION GET_VALUE   (sSession_ID varchar2,sCode varchar2) RETURN varchar2;
  FUNCTION GET_VALUE_N (sSession_ID varchar2,sCode varchar2) RETURN number;
  
  FUNCTION GET_USER_SID(sSession_ID varchar2) RETURN number;
  FUNCTION GET_MANDANT(sSession_ID varchar2) RETURN number;
END;
/

------------------------------------------------------------------

CREATE OR REPLACE PACKAGE BODY BLV_SESSION IS

  FUNCTION GET_VERSION RETURN VARCHAR2 IS
  BEGIN
    RETURN '1.01';
  END;
  
  /** Session-Initialisierung 
   *  bergaben:
   *    sSession_ID: Eindeutiges identifizierungs-Literal
   *    nUserSID:    BALVI iP UserSID (S50.SID), welches beim Login ermittelt wurde
   *  Beschreibung:
   *     INIT ruft implizit auch CLEAR() auf, um ggf. alte Werte zu lschen.
   *     Fr die neue Session_ID wird der Kopfsatz mit dem CODE='SESSION' angelegt,
   *     VALUE    = NULL
   *     VALUE_DF = SYSDATE der Anlage (Session-Start)
   *     VALUE_DT = SYSDATE +1 (Session-Ende), wenn nicht explizit gelscht
   *     VALUE_N  = nUserSID
   **/
  PROCEDURE INIT (sSession_ID varchar2, nUserSID number, sSessionName varchar2 := null) IS
  /* Variablen */
  /* Body */
  BEGIN
    BLV_SESSION.CLEAR(sSession_ID);
    BLV_SESSION.ADD(sSession_ID,'SESSION',sSessionName,nUserSID,NULL,SYSDATE,SYSDATE+1);
    BLV_SESSION.ADD(sSession_ID,'USER',nUserSID);
    BLV_SESSION.ADD_ZB_FILTER(sSession_ID,nUserSID);
    BLV_SESSION.ADD_USER_INFO(sSession_ID,nUserSID);
  END;
  
  /** CLEAR - Lsche komplette Session
   *  bergaben:
   *    sSession_ID: Eindeutiges identifizierungs-Literal
   **/
  PROCEDURE CLEAR (sSession_ID varchar2) is
  BEGIN
    DELETE FROM WKD_SESSION WHERE SESSION_ID = sSession_ID;
    COMMIT;
  END;
  
  /** ADD - Globale Hauptfunktion
   *  bergaben:
   *    sSession_ID: Eindeutiges identifizierungs-Literal
   *    sCode:       Name des Session-Parameters IMMER UPPERCASE
   *    sValue:      Wert als String (in Spalte WKD_SESSION.VALUE)
   *    nValue:      Wert als Integer (in Spalte WKD_SESSION.VALUE_N)
   *    fValue:      Wert als Float (in Spalte WKD_SESSION.VALUE_F), kein "simples ADD" vorhanden
   *    dValueFrom:  Wert als Datum (in Salte WKD_SESSION.VALUE_DF)
   *    dValueTo:    Wert als Datum (in Salte WKD_SESSION.VALUE_DT)
   *    lfdnr:       Integer fr Sortier-Reihenfolgen, default 1
   *
   **/
  PROCEDURE ADD(sSession_ID varchar2, sCode varchar2, sValue varchar2, nValue number, fValue float, dValueFrom date, dValueTo date, lfdnr number) is
  /* Variablen */
  /* Body */
  BEGIN
  	INSERT INTO WKD_SESSION (ID, SESSION_ID,  CODE,  VALUE, VALUE_N, VALUE_F, VALUE_DF, VALUE_DT, LFDNR ) 
           VALUES ( SYS_GUID(), sSession_ID, UPPER(sCode), sValue, nvalue,  fValue, dValueFrom, dValueTo, lfdnr );
    COMMIT;
  END;
  
  /** ADD - OVERLOAD-Methoden
   *
   **/
   
  PROCEDURE ADD(sSession_ID varchar2, sCode varchar2, sValue varchar2, nValue number, fValue float, dValueFrom date, dValueTo date) is
  BEGIN
  	BLV_SESSION.ADD(sSession_ID,sCode,sValue,nValue,fvalue,dValueFrom,dValueTo,1);
  END;
  
  PROCEDURE ADD(sSession_ID varchar2, sCode varchar2, sValue varchar2) is
  BEGIN
    BLV_SESSION.ADD(sSession_ID,sCode,sValue,NULL,NULL,NULL,NULL);
  END;
  
  PROCEDURE ADD(sSession_ID varchar2, sCode varchar2, nValue number) is
  BEGIN
    BLV_SESSION.ADD(sSession_ID,sCode,NULL,nValue,NULL,NULL,NULL);
  END;
  
  /*
  PROCEDURE ADD(sSession_ID varchar2, sCode varchar2, fValue float) is
  BEGIN
    BLV_SESSION.ADD(sSession_ID,sCode,NULL,NULL,fValue,NULL,NULL);
  END;
  */
  
  PROCEDURE ADD(sSession_ID varchar2, sCode varchar2, dValueFrom date, dValueTo date) is
  BEGIN
    BLV_SESSION.ADD(sSession_ID,sCode,NULL,NULL,NULL,dValueFrom,dValueTo,1);
  END;
  
  /** UPD - Aktualisierung eines Wertes
   *
   *  bergaben
   *    => Siehe ADD()
   *    addIfNull BOOLEAN = TRUE oder FALSE bergeben, default TRUE
   *  Besonderheiten:
   *    Die Funktion UPD() lscht alle nicht bergebenen Spalten
   *    aus dem Datensatz. Wenn der Datensatz mit dem entsprechenden CODE
   *    1.) NICHT EINDEUTIG => ROWCOUNT > 1 ist,
   *         => keine ndern
   *    2.) NICHT VORHANDEN => ROWCOUNT = 0 ist,
   *         => es wird ADD() aufgerufen, wenn addIfNull=TRUE 
   **/
  PROCEDURE UPD(sSession_ID varchar2, sCode varchar2, sValue varchar2, nValue number := NULL , fValue float := NULL , dValueFrom date := NULL, dValueTo date := NULL, lfdnr number := 1, addIfNull boolean := TRUE) IS
  /* Variablen */ 
    CURSOR C_WKD_SESSION is
      SELECT WKD_SESSION.*,ROWID FROM WKD_SESSION
      WHERE SESSION_ID = sSession_ID AND CODE = sCode 
      FOR UPDATE;
    D_WKD_SESSION  C_WKD_SESSION%ROWTYPE;
  /* Body */
  BEGIN
  	OPEN C_WKD_SESSION;
    FETCH C_WKD_SESSION INTO D_WKD_SESSION;
    
  	IF C_WKD_SESSION%NOTFOUND AND C_WKD_SESSION%ROWCOUNT=0 AND addIfNull THEN
  	  CLOSE C_WKD_SESSION;
      --Satz nicht da -> Anlegen
      BLV_SESSION.ADD(sSession_ID,sCode,sValue,nValue,fValue,dValueFrom,dValueTo,lfdnr);
      RETURN;
    END IF;
    
    IF C_WKD_SESSION%ROWCOUNT>1 THEN
      -- Abbruch, da nicht eindeutig
      CLOSE C_WKD_SESSION;
      RETURN;
    END IF;
    
    --EXIT WHEN C_WKD_SESSION%NOTFOUND;
    UPDATE WKD_SESSION SET 
      VALUE = sValue, VALUE_N = nValue, VALUE_F = fValue, VALUE_DF = dValueFrom, VALUE_DT = dValueTo
    WHERE ROWID = D_WKD_SESSION.ROWID;
    COMMIT;
    CLOSE C_WKD_SESSION;
  END;
  
  PROCEDURE DEL(sSession_ID varchar2, sCode varchar2) IS
  /* Variablen */ 
    CURSOR C_WKD_SESSION is
      SELECT WKD_SESSION.*,ROWID FROM WKD_SESSION
      WHERE SESSION_ID = sSession_ID AND CODE = sCode 
      FOR UPDATE;
    D_WKD_SESSION  C_WKD_SESSION%ROWTYPE;
  /* Body */
  BEGIN
  	OPEN C_WKD_SESSION;
    
    LOOP    
      FETCH C_WKD_SESSION INTO D_WKD_SESSION;
      EXIT WHEN C_WKD_SESSION%NOTFOUND;
      DELETE FROM WKD_SESSION WHERE ROWID = D_WKD_SESSION.ROWID;
    END LOOP;
    COMMIT;
    CLOSE C_WKD_SESSION;
  END;
  
  FUNCTION GET_VALUE_N(sSession_ID varchar2,sCode varchar2) RETURN number IS
  RESULT NUMBER;
  CURSOR C_WKD_SESSION IS
    SELECT VALUE_N FROM WKD_SESSION
    WHERE SESSION_ID = sSession_ID
      AND CODE = sCode;
    D_WKD_SESSION C_WKD_SESSION%ROWTYPE;
  BEGIN
  	OPEN C_WKD_SESSION;
  	RESULT := NULL;
  	LOOP
  	  FETCH C_WKD_SESSION INTO D_WKD_SESSION;
  	  EXIT WHEN C_WKD_SESSION%NOTFOUND;
  	  RESULT := D_WKD_SESSION.VALUE_N;
  	END LOOP;
  	CLOSE C_WKD_SESSION;
  	RETURN RESULT;
  END;
  
  FUNCTION GET_VALUE(sSession_ID varchar2,sCode varchar2) RETURN varchar2 IS
  RESULT VARCHAR2(4000);
  CURSOR C_WKD_SESSION IS
    SELECT VALUE FROM WKD_SESSION
    WHERE SESSION_ID = sSession_ID
      AND CODE = sCode;
    D_WKD_SESSION C_WKD_SESSION%ROWTYPE;
  BEGIN
  	OPEN C_WKD_SESSION;
  	RESULT := NULL;
  	LOOP
  	  FETCH C_WKD_SESSION INTO D_WKD_SESSION;
  	  EXIT WHEN C_WKD_SESSION%NOTFOUND;
  	  RESULT := D_WKD_SESSION.VALUE;
  	END LOOP;
  	CLOSE C_WKD_SESSION;
  	RETURN RESULT;
  END;
  
  FUNCTION GET_MANDANT(sSession_ID varchar2) RETURN number IS
  BEGIN
  	RETURN GET_VALUE_N(sSession_ID,'MANDANT');
  END;
  
  FUNCTION GET_USER_SID(sSession_ID varchar2) RETURN number IS
  BEGIN
  	RETURN GET_VALUE_N(sSession_ID,'USER');
  END;
  
  PROCEDURE ADD_ZB_FILTER(sSession_ID varchar2,nUserSID number) IS
  /* Variablen */
    CURSOR C_ZB_USER IS
      SELECT
        A25.CODE    AS CODE
       ,A25.ZB      AS ZB
       ,A25.BEZ     AS BEZ
       ,A25.ID      AS ID_VALUE
       ,ROWNUM      AS LFDNR
       ,ZB_USER.AL  AS AL
     FROM
       GLO_A25 A25
     ,(SELECT 
             A25_R.SID
            ,S5002.AL
            ,RANK() OVER (PARTITION BY A25_R.CODE ORDER BY LENGTH(A25_Z.CODE) DESC NULLS LAST) RANK
            FROM 
              VIS_S5002 S5002
             ,GLO_A25   A25_Z
             ,GLO_A25   A25_R
            WHERE
              S5002.S50_SID = CASE WHEN EXISTS (SELECT 'x' 
                                                FROM VIS_S5002 
                                                WHERE DELETED = 0 AND S50_SID = nUserSID)
                                   THEN nUserSID
                                   ELSE (SELECT S50_SID00 FROM VIS_S50 WHERE SID = nUserSID)
                              END 
              AND S5002.GLO_A25_SID = A25_Z.SID
              AND S5002.DELETED = 0
              AND A25_Z.DELETED = 0
              AND A25_R.ZB LIKE A25_Z.ZB || '%'
              AND A25_R.KAT <= 5
              AND A25_R.DELETED = 0
    ) ZB_USER
    WHERE
      ZB_USER.SID = A25.SID
      AND ZB_USER.RANK = 1
      AND ZB_USER.AL > 1 -- Berechtigungen auf LESEN oder hher (2,3,4)
    ORDER BY
      A25.ZB
    ;
    D_ZB_USER  C_ZB_USER%ROWTYPE;
  /* Body */
  BEGIN
  	OPEN C_ZB_USER;
  	-- SANITY CHECK: CLEAR fr ZB-Filter
    BLV_SESSION.DEL(sSession_ID,'ZB_FILTER');
    LOOP    
      FETCH C_ZB_USER INTO D_ZB_USER;
      EXIT WHEN C_ZB_USER%NOTFOUND;
                   -- sSession_ID, sCode, sValue, nValue, fValue, dValueFrom, dValueTo, lfdnr
      BLV_SESSION.ADD(sSession_ID,'ZB_FILTER', D_ZB_USER.CODE, D_ZB_USER.AL, NULL,NULL,NULL,D_ZB_USER.LFDNR);
    END LOOP;
    COMMIT;
    CLOSE C_ZB_USER;
  END;
  
  PROCEDURE ADD_USER_INFO(sSession_ID varchar2,nUserSID number) IS
  /* Variablen */
    CURSOR C_USER IS
      SELECT 
         S50.D0102_ID
        ,CASE WHEN D010200.BEZ IS NULL THEN S50.BEZ
              ELSE D010200.BEZ
         END AS USER_BEZ
        ,S50.CODE AS USER_CODE
        ,D010200.ID AS D010200_ID
        ,D010200.KOM_EMAIL AS USER_EMAIL
        ,S50.S50_SID00 AS MANDANT_SID
        ,D0101.ID AS D0101_ID
        ,S50_00.CODE AS MANDANT_CODE
        ,S50_00.BEZ AS MANDANT_BEZ
      FROM 
        VIS_S50     S50
          LEFT JOIN VIS_D0102 D0102 ON S50.D0102_ID = D0102.ID AND D0102.DELETED > -2
          INNER JOIN VIS_S50 S50_00 ON S50.S50_SID00 = S50_00.SID AND S50_00.SKZ = 1
            INNER JOIN VIS_D0101 D0101 ON S50_00.D0101_ID = D0101.ID
            LEFT JOIN VIS_D010200 D010200 ON D010200.D0101_ID = S50_00.D0101_ID AND D010200.D0102_ID = D0102.ID AND D010200.DELETED > -2
      WHERE S50.SKZ = 3 -- Nur Benutzer
        AND S50.DELETED = 0 -- nicht gelscht
        AND (S50.PASSWORT IS NOT NULL OR S50.AD_USER_GUID IS NOT NULL) -- Anmeldung erlaubt
        AND  S50.SID = nUserSID  -- UserSID
    ;
    D_USER  C_USER%ROWTYPE;
  /* Body */
  BEGIN
  	OPEN C_USER;
  	  LOOP
  	    FETCH C_USER INTO D_USER;
  	    EXIT WHEN C_USER%NOTFOUND;
  	    -- Werte einfgen (mit vorherigem Lschen):
  	    BLV_SESSION.DEL(sSession_ID,'USER_CODE');
  	    BLV_SESSION.ADD(sSession_ID,'USER_CODE', D_USER.USER_CODE);
  	    BLV_SESSION.DEL(sSession_ID,'USER_BEZ');
  	    BLV_SESSION.ADD(sSession_ID,'USER_BEZ', D_USER.USER_BEZ);
  	    BLV_SESSION.DEL(sSession_ID,'USER_EMAIL');
  	    BLV_SESSION.ADD(sSession_ID,'USER_EMAIL', D_USER.USER_EMAIL);
  	    BLV_SESSION.DEL(sSession_ID,'D0102_ID');
  	    BLV_SESSION.ADD(sSession_ID,'D0102_ID', D_USER.D0102_ID);
  	    BLV_SESSION.DEL(sSession_ID,'D010200_ID');
  	    BLV_SESSION.ADD(sSession_ID,'D010200_ID', D_USER.D010200_ID);
  	    BLV_SESSION.DEL(sSession_ID,'D0101_ID');
  	    BLV_SESSION.ADD(sSession_ID,'D0101_ID', D_USER.D0101_ID);
  	    BLV_SESSION.DEL(sSession_ID,'MANDANT');
  	    BLV_SESSION.ADD(sSession_ID,'MANDANT', D_USER.MANDANT_SID);
  	    BLV_SESSION.DEL(sSession_ID,'MANDANT_CODE');
  	    BLV_SESSION.ADD(sSession_ID,'MANDANT_CODE', D_USER.MANDANT_CODE);
  	    BLV_SESSION.DEL(sSession_ID,'MANDANT_BEZ');
  	    BLV_SESSION.ADD(sSession_ID,'MANDANT_BEZ', D_USER.MANDANT_BEZ);
  	  END LOOP;
  	CLOSE C_USER;
  END;
  
  PROCEDURE ADD_S07_PARAM(sSession_ID varchar2 ,nUserSID number, sS07_CODE varchar2, sS0701_CODE varchar2, sCode varchar2 := NULL) IS
  /* Variablen */
  CURSOR C_S07_USER IS
    SELECT SYS_GUID() ID
      , d1.s07_id
      , d1.s0701_id
      , d1.s0702_id
      , d1.s07_code
      , d1.s07_bez
      , d1.s0701_code
      , d1.s0701_bez
      , d1.auswert_level
      , d1.typ
      , d1.mandant
      , d1.s50_03_sid
      , d1.fehlernr
      , CASE WHEN Fehlernr = 0 THEN wert ELSE NULL END wert 
      , CASE d1.fehlernr
          WHEN 15 THEN 'Keine Benutzerwert vorhanden.'
          WHEN 16 THEN 'Keine Mandantenwert vorhanden.'
          WHEN 17 THEN 'Fr Mandantenparameter ist ein Benutzerwert vorhanden.'
          WHEN 18 THEN 'Fr Benutzerparameter ist ein Mandantenwert vorhanden.'
          WHEN 19 THEN 'Kein Benutzer-/Mandantenparameter vorhanden.'
          WHEN 13 THEN 'Kein Wert vorhanden.'
          WHEN 14 THEN 'Kein Wert erfasst.'   
          WHEN 20 THEN 'Systemparameter ist mit Mandanten- bzw. Benutzerzuordnung erfasst.'   
          ELSE NULL
        END Fehlertext
      , d1.rank
    FROM (
          SELECT s07.id   s07_id
            , s0701.id    s0701_id
            , s0702.id    s0702_id 
            , s07.code    s07_code
            , s07.bez     s07_bez
            , s0701.code  s0701_code
            , s0701.bez   s0701_bez
            , S0701.auswert_level
            , s0701.typ
            , s0702.mandant
            , s0702.s50_03_sid
            , s0702.wert wert
            , s07.deleted s07_deleted
            , RANK() OVER (PARTITION BY s0701.id ORDER BY s0702.s50_03_sid DESC, s0702.mandant DESC NULLS LAST) rank
            , CASE 
                WHEN s0701.auswert_level LIKE 'U' AND s0702.mandant > 0 AND S0702.s50_03_sid < 0 THEN 18 --'Fr Benutzerparameter ist ein Mandantenwert vorhanden'
                WHEN s0701.auswert_level LIKE 'U' AND ( s0702.s50_03_sid IS NULL OR S0702.s50_03_sid < 0) THEN 15 --'Keine Benutzerwert vorhanden'
                WHEN s0701.auswert_level LIKE 'M' AND ( s0702.mandant IS NULL OR S0702.mandant < 0) THEN 16 --'Keine Mandantenwert vorhanden'
                WHEN s0701.auswert_level LIKE 'M' AND ( s0702.s50_03_sid > 0) THEN 17 --'Fr Mandantenparameter ist ein Benutzerwert vorhanden'
                WHEN s0701.auswert_level NOT LIKE '%S%' AND s0701.auswert_level LIKE '%U%' AND s0701.auswert_level LIKE '%M%' AND s0702.mandant = -1 AND s0702.s50_03_sid = -1 THEN 19 --'Kein Benutzer-/Mandantenparameter vorhanden.'
                WHEN S0702.id IS NULL THEN   13 --'Kein Wert vorhanden' 
                WHEN S0702.WERT IS NULL THEN 14 -- 'Kein Wert erfat'   
                WHEN s0701.auswert_level LIKE 'S' AND  ( s0702.mandant > -1 OR s0702.s50_03_sid > -1) THEN 20
                ELSE 0
              END Fehlernr
          FROM vis_s07 s07
          JOIN vis_s0701 s0701 ON s0701.s07_id = s07.id
            AND s07.deleted = 0 
            AND s0701.deleted = 0
          LEFT JOIN vis_s0702 s0702 ON s0702.s0701_id = s0701.id 
            AND s0702.deleted = 0
            AND ( 
                  (s0702.mandant = BLV_SESSION.GET_MANDANT(sSession_ID)
                   AND NVL(s0702.s50_03_sid, -1) = -1 
                   AND (s0701.auswert_level LIKE '%M%' OR s0701.auswert_level = '%' )
                   ) OR
                  (s0702.s50_03_sid = BLV_SESSION.GET_USER_SID(sSession_ID)
                   AND s0702.mandant = BLV_SESSION.GET_MANDANT(sSession_ID)
                   AND (s0701.auswert_level LIKE '%U%' OR s0701.auswert_level = '%' )
                   ) OR
                  ((s0701.auswert_level LIKE '%S%' OR s0701.auswert_level = '%') 
                    AND NVL(s0702.mandant,-1) = -1 AND NVL(s0702.s50_03_sid, -1) = -1 )
                )  
          ORDER BY s0701.code
    ) d1
    WHERE D1.RANK = 1
      AND UPPER(D1.S07_CODE) = UPPER(sS07_CODE)
      AND UPPER(D1.S0701_CODE) = UPPER(sS0701_CODE)
    ;
    D_S07_USER C_S07_USER%ROWTYPE;
    sCodeNew VARCHAR2(128);
  /* Body */
  BEGIN
  	OPEN C_S07_USER;
  	LOOP
  	    FETCH C_S07_USER INTO D_S07_USER;
  	    EXIT WHEN C_S07_USER%NOTFOUND;
  	    -- TODO: Werte einfgen
  	    IF ( D_S07_USER.FEHLERNR = 0) THEN
  	       IF sCode IS NULL THEN 
  	          sCodeNew := sS07_CODE || '.' || sS0701_CODE;
           ELSE
              sCodeNew := sCode;
  	       END IF;
  	       -- TYP: 0 - String (Standard), 1 - Integer, 3 -Boolean, 5 - Passwort, 6,7 - Match+ MatchMulti, 11 - Verzeichnis, 12 - Dateiname, 13 - Dateimaske
  	       IF (D_S07_USER.TYP = 1 OR D_S07_USER.TYP = 3) AND ISNUMERIC(D_S07_USER.WERT)=1 THEN
             BLV_SESSION.ADD(sSession_ID,sCodeNew, CAST(D_S07_USER.WERT AS NUMBER ) );
           ELSE 
             -- Standard-Fall = STRING
             BLV_SESSION.ADD(sSession_ID,sCodeNew, D_S07_USER.WERT);
           END IF; 
  	    END IF;
  	END LOOP;
  	CLOSE C_S07_USER;
  END;
  
  /** is_Valid - berprfung, ob Session/User gltig ist
   *  Fallbeispiel: User ndert sich, Session_id bleibt (Benutzerwechsel)
   *  Rckgaben: 
   *    0 = FALSE
   *    1 = TRUE
   **/
  FUNCTION is_Valid(sSession_ID varchar2,nUserSID number) RETURN number IS
  RESULT NUMBER;
  CURSOR C_WKD_SESSION IS
    SELECT VALUE_N FROM WKD_SESSION
    WHERE SESSION_ID = sSession_ID
      AND CODE = 'SESSION';
    D_WKD_SESSION C_WKD_SESSION%ROWTYPE;
  BEGIN
  	OPEN C_WKD_SESSION;
  	RESULT := 0;
  	LOOP
  	  FETCH C_WKD_SESSION INTO D_WKD_SESSION;
  	  EXIT WHEN C_WKD_SESSION%NOTFOUND;
  	  IF D_WKD_SESSION.VALUE_N = nUserSID AND C_WKD_SESSION%ROWCOUNT=1 THEN
  	    RESULT := 1;
  	  ELSE
  	    RESULT := 0;
  	  END IF;
  	END LOOP;
  	CLOSE C_WKD_SESSION;
  	RETURN RESULT;
  END;
  
END;
/
